
Texture2D g_TextureColor;
Texture2D g_TextureNormal;

float3 g_CameraPosition;

float  g_Time;

matrix g_World;
matrix g_ViewProjection;
matrix g_WorldViewProjection;

bool g_SamplerLow = false;

SamplerState sam
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VS_INPUT
{
    float3 pos		: POSITION;
    float3 normal	: NORMAL;
    float2 tex		: TEXCOORD0;
};

struct PS_INPUT
{
    float4 posP			: SV_POSITION;
    float4 posW			: TEXCOORD1;
    float3 normalW		: NORMAL;
    float2 tex			: TEXCOORD0;
};




PS_INPUT VS( VS_INPUT input)
{
    PS_INPUT output;
    
    float3 tempPos = input.pos;
    
    tempPos = (tempPos - float3(0.5,0,0.5))*100 + float3(g_CameraPosition.x, 0, g_CameraPosition.z);
    
    output.posW = mul( float4(tempPos,1), g_World );    
    output.posP = mul( output.posW, g_ViewProjection );
	
    output.tex = (input.tex + (float2(g_CameraPosition.x,g_CameraPosition.z) * 0.01)) * 20;
    output.normalW = mul( float3(0,1,0) , (float3x3)g_World );

    return output;
}




void PS( PS_INPUT input , 
		   out float4 color  : SV_Target0,
		   out float4 normal : SV_Target1,
		   out float4 pos	 : SV_Target2,
		   out float4 pp_normal	 : SV_Target3,
		   out float4 info : SV_Target4 )
{    
	float2 temptex = input.tex;
	
	temptex.x = temptex.x - (float)((int)temptex.x) + 1;
	temptex.y = temptex.y - (float)((int)temptex.y) + 1;
	temptex.x = temptex.x - (float)((int)temptex.x);
	temptex.y = temptex.y - (float)((int)temptex.y);
    
	pos = input.posW;
	
		normal = float4(normalize(normalize(input.normalW) * (g_TextureNormal.Sample(sam, temptex).rgb )),1);
		color = float4(g_TextureColor.Sample(sam, temptex).rgb,1);
		
    pp_normal = float4(0,0,0,0);
    info = float4(input.posP.z, 0, 0, 1);
}



technique10 Default
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}