
struct LIGHTDATA
{
	int				Type;

	float4			Diffuse;
	float4			Specular;
	float4			Ambient;

	float3			Position;
	float3			Direction;

	float           Range;

	float           Attenuation0;
	float           Attenuation1;
	float           Attenuation2;
};


Texture2D g_texColor;
Texture2D g_texNormal;
Texture2D g_texPos;
Texture2D g_texDepth;



LIGHTDATA g_Lights[400];
int g_LightCount = 0;

float3 g_MaterialAmbient   = float3( 0.2f, 0.2f, 0.2f );
float3 g_MaterialDiffuse   = float3( 0.8f, 0.8f, 0.8f );
float3 g_MaterialSpecular  = float3( 1.0f, 1.0f, 1.0f );
int    g_MaterialShininess = 32;
float3 g_CameraPosition;
bool   g_Wireframe = false;
bool   g_AmbientOnly = false;


SamplerState samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};








float4 calcBlinnPhongLighting( float3 n, float3 p, float4 color )
{
	float3 v = normalize(g_CameraPosition - p);
	float4 sumcolor = float4(0,0,0,0);
	
	float3 l;
	float3 h;
	float4 Ia;
	float4 Id;
	float4 Is;
	float3 dist;
	float dist_l;
	
	for (int i = 0 ; i < g_LightCount ; i ++)
	{
		dist = p - g_Lights[i].Position;
		dist_l = length(dist);
		
		if ( dist_l < g_Lights[i].Range )
		{	
			if ( g_AmbientOnly )
			{
			    Ia = float4(g_MaterialAmbient,1);
			    
			    sumcolor.rgb += Ia.rgb * g_Lights[i].Ambient  * color.rgb;
			}
			else
			{
			    l = g_Lights[i].Direction;
			    if (g_Lights[i].Type == 1) l = dist;
			    l = normalize(l);
    			
			    h = normalize(-l + v);
			
			    Id = float4(g_MaterialDiffuse,1) * saturate( dot(n,-l) );
			    Is = float4(g_MaterialSpecular,1) * pow( saturate(dot(n,h)), g_MaterialShininess );
			    
			    sumcolor.rgb += (Id * g_Lights[i].Diffuse + Is * g_Lights[i].Specular) * color.rgb  / (g_Lights[i].Attenuation0 + 
																								      (g_Lights[i].Attenuation1*dist_l) + 
																								      (g_Lights[i].Attenuation2*dist_l*dist_l) );
			}
		}
	}
	
	sumcolor.a = 1;
	
    return sumcolor;
}





void PS(  in	float4 pos		: SV_POSITION,
		  in	float2 tex		: TEXCOORD0,
		  out	float4 col		: SV_TARGET,
		  out   float depth     : SV_DEPTH )
{
    float4 color = g_texColor.Sample( samLinear, tex );
    float4 position = g_texPos.Sample( samLinear, tex );
    float4 normal = g_texNormal.Sample( samLinear, tex );
    
    depth = g_texDepth.Sample( samLinear ,tex ).x;
    
    col = calcBlinnPhongLighting(normal, position, color);
}

void VS( inout		float4 pos		: SV_POSITION,
		 inout		float2 tex		: TEXCOORD0) 
{ 
	tex = (pos.xy + float2(1,1)) / 2;
	tex.y = tex.y*-1;
}









RasterizerState rs
{ 
	CullMode = None; 
	FillMode = Solid;
};
DepthStencilState RenderNonShadows
{
    DepthEnable = true;
    DepthWriteMask = ALL;
    DepthFunc = Less;
    
    StencilEnable = true;
    StencilReadMask = 0xFFFFFFFF;
    StencilWriteMask = 0x0;
    
    FrontFaceStencilFunc = Equal;
    FrontFaceStencilPass = Keep;
    FrontFaceStencilFail = Zero;
    
    BackFaceStencilFunc = Never;
    BackFaceStencilPass = Zero;
    BackFaceStencilFail = Zero;
};
DepthStencilState RenderNonShadows_NoDepth
{
    DepthEnable = false;
    
    StencilEnable = true;
    StencilReadMask = 0xFFFFFFFF;
    StencilWriteMask = 0x0;
    
    FrontFaceStencilFunc = Equal;
    FrontFaceStencilPass = Keep;
    FrontFaceStencilFail = Zero;
    
    BackFaceStencilFunc = Never;
    BackFaceStencilPass = Zero;
    BackFaceStencilFail = Zero;
};

BlendState AdditiveBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    SrcBlend = ONE;
    DestBlend = ONE;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

technique10 LightingDiffuseAndSpecular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        
		SetRasterizerState( rs );
		SetDepthStencilState(RenderNonShadows_NoDepth,0);
        SetBlendState( AdditiveBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}

BlendState StdBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

technique10 LightingAmbient
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        
		SetRasterizerState( rs );
		SetDepthStencilState(RenderNonShadows,0);
        SetBlendState( StdBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}